/*
 *  VARIABLES, {akii-root}/src/variables.c
 *
 * ==========================================================================
 *
 *                      Arena Kliens II. Releases
 *
 * ==========================================================================
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * A legkiralyabb szovegszerkeszto: VIM - Vi IMproved 5  <http://www.vim.org>
 */

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "error.h"
#include "variables.h"
#include "commands.h"
#include "expression.h"
#include "interpreter.h"
#include "screen.h"
#include "changer.h"
#include "main.h"
#include "debug.h"
#include "terms.h"

#ifdef abs
#undef abs(x)
#endif
#define abs(x)	(((x)<0)?-(x):(x))

struct valtozo *varlist = NULL /*,*varlist_b=NULL */ ;
struct valtozo *bivarlist = NULL /*,*bivarlist_b=NULL */ ;
struct valtozo *localvars[1024];
char **numvarlist = NULL;
char **chvarlist = NULL;
char **mchvarlist = NULL;
int kerdojel = 0, mkerdojel = 0;
int locallevel = 0;

struct onstr *onlist = NULL;
struct onstr *onsendlist = NULL;

struct timerstr *timerlist = NULL;

struct valtozo *akii_prompt, *akii_sortby, *akii_termwidth, *akii_termheight;
struct valtozo *akii_winsize, *akii_usewin, *akii_idle, *akii_inputidle,
 *akii_floodprotect;
char sortby = 'N';

char *blockname=NULL;

char *use_list[100];
int usenum = 0;
int delquote = 0;
int dontquote = 0;
int varflag = VFLAG_NORMAL;

/*valtozo kinyerese */
struct valtozo *getvar(struct valtozo **hol, char **s, int makeifnotexist, int lehetstruct);

struct valtozo *getvarifp(struct valtozo **hol, char **s);

/*szamozott valtozo kinyerese (uj stringnek foglal helyett) */
char *getnumvar(char **itt, int elos, int utolso);

/*visszater a mar behelyettesitett szovegel */
char *parsedollar(char *s);

/*visszater, hogy mennyit rakott `ide' :-) */
int dollar(char **s, char *ide);

/*a kovetkezo zarojelig kiemelei a szoveget */
void braces(char **p, char *ide, char brace1, char brace2);

/*valtozo torlese (strukturae is) */
int freevarlist(struct valtozo *hol, int action);

/*numvar lista torlese */
void freenumvarlist(char ***s);

/*initvariables: belso valtozok... */
void initvariables(char *server, int port, char *nev, char *verinfo, char *ver, int builtno);

/*donevariables_fvs */
void donevariables_fvs(void);

/* egy valtozo beallitasa */
struct valtozo *setvar(struct valtozo *ezt, char *erre);

/* uj belso fv beallitasa */
struct valtozo *newfv(char *ezt, void (*fv) (struct valtozo *));

char *parsedollar(char *s)
{				/*uj stringnek foglal helyet!!! */
    char temp[1024];
    char *p = s;
    int i = 0, x;

#ifdef AKII_DEBUG
    dbgenter(dbg_parsedollar, s);
#endif
    if (!p) {
	p = malloc(1);
	*p = 0;
#ifdef AKII_DEBUG
	dbgleave(dbg_parsedollar);
#endif
	return (p);
    }
    while (1) {
	while ((*p) && (*p != '$'))
	    temp[i++] = *p++;
	if (!*p)
	    break;
	if ((p != s) && (*(p - 1) == '\\')) {
	    temp[i - 1] = '$';
	    p++;
	}
	else {
	    x = dollar(&p, temp + i);
	    if (x < 0) {
		if (errno != ENOVAR)
		    break;
		else
		    errno = 0;
	    }
	    else
		i += x;
	}
    }
    temp[i] = 0;
    p = malloc(i + 1);
    strcpy(p, temp);
#ifdef AKII_DEBUG
    dbgleave(dbg_parsedollar);
#endif
    return (p);
}

void braces(char **p, char *ide, char brace1, char brace2)
{
    int x = 1, y = 0;
    int i = 0;
    int sl = 0;

#ifdef AKII_DEBUG
    dbgenter(dbg_braces, *p);
#endif
    while (**p) {
	if (**p == '\\') {
	    sl ^= 1;
	    if (!sl) {
		ide[i++] = **p;
		if (!delquote)
		    ide[i++] = '\\';
	    }
	    (*p)++;
	    continue;
	}
	if (brace1 != '"')
	    if (**p == '"')
		if (!sl) {
		    y ^= 1;
		}
	if ((**p == brace1) && (brace1 != '"'))
	    if (!y) {
		if (!sl)
		    x++;
	    }
	if (**p == brace2)
	    if (!y) {
		if (!sl) {
		    if (!--x)
			break;
		}
	    }
	if (sl) {
	    if (!((**p == brace1) && (delquote)))
		ide[i++] = '\\';
	}
	ide[i++] = **p;
	(*p)++;
	sl = 0;
    }
    if (sl)
	ide[i++] = '\\';
    ide[i] = 0;
#ifdef AKII_DEBUG
    dbgleave(dbg_braces);
#endif
    return;
}
char *gettoretlen(int elso, int utolso)
{
    char *s, *p = toretlenargs;
    int i = 0, j = 0, szoban = 0, a = 0, sl = 0;
    int k = 0, pakolj = 0;

#ifdef AKII_DEBUG
    dbgenter(dbg_gettoretlen, "?");
#endif
    if (!toretlenargs) {
	s = malloc(1);
	*s = 0;
#ifdef AKII_DEBUG
	dbgleave(dbg_gettoretlen);
#endif
	return (s);
    }
    s = malloc(strlen(toretlenargs) + 1);
    memset(s, 0, strlen(toretlenargs) + 1);
    while (p[i]) {
	if (p[i] == '\\') {
	    sl ^= 1;
	    if (!sl)
		if (pakolj) {
		    s[j++] = '\\';
		    s[j++] = '\\';
		}
	    i++;
	    continue;
	}
	if ((p[i] == '"') && (!sl)) {
	    a ^= 1;
	    i++;
	    continue;
	}
	if ((!szoban) && (!isspace(p[i]))) {
	    szoban = 1;
	    k++;
	    if (k == elso)
		pakolj = 1;
	}
	if (isspace(p[i]) && (!a)) {
	    szoban = 0;
	    if (k == utolso)
		break;
	}
	if (sl)
	    if (pakolj)
		s[j++] = '\\';
	sl = 0;
	if (pakolj)
	    s[j++] = p[i];
	i++;
    }
    s[j] = 0;
    s = realloc(s, j + 1);
#ifdef AKII_DEBUG
    dbgleave(dbg_gettoretlen);
#endif
    return (s);
}
char *dollarnum(char **s, int hol)
{				/*hol=(0=numvarlist),(1=chvarlist),(2=mchvarlist),(3=keresd meg magad!) */
    char *p = *s, *t = *s;
    char *w;
    int elso, utolso;

    if (*t == '-') {
	utolso = strtol(++p, &t, 10);
	if (p == t) {
	    strcpy(errtemp, (*s));
	    *s = p;
	    errno = ENONUM;
	    return (NULL);
	}
	elso = 1;
	*s = t;
	if (hol == 0)
	    w = getnumvar(numvarlist, 1, utolso);
	else if (hol == 1)
	    w = getnumvar(chvarlist, 1, utolso);
	else if (hol == 2)
	    w = getnumvar(mchvarlist, 1, utolso);
	else
	    w = gettoretlen(1, utolso);
	return (w);
    }
    else if (isdigit(*t)) {
	utolso = elso = strtol(p, &t, 10);
	p = t;
	if (*t == '-') {
	    if (!isdigit(*++p))
		utolso = -1;
	    else {
		utolso = strtol(p, &t, 10);
		p = t;
		if (utolso < elso) {
		    t = malloc(1);
		    *t = 0;
		    return (t);
		}
	    }
	}
	*s = p;
	if (hol == 0)
	    w = getnumvar(numvarlist, elso, utolso);
	else if (hol == 1)
	    w = getnumvar(chvarlist, elso, utolso);
	else if (hol == 2)
	    w = getnumvar(mchvarlist, elso, utolso);
	else
	    w = gettoretlen(elso, utolso);
	return (w);
    }
    return (NULL);
}
char *igazit(char *s, int igazitas, int kozepre)
{
    int i, x, y, c;
    char *ide;

    if (kozepre >= 10) {
	igazitas = 0;
	kozepre -= 10;
    }
    if (!igazitas && kozepre) {
	switch (kozepre) {
	    case 1:
		igazitas = COLS - 1;
		break;
	    case 2:
		igazitas = -(COLS - 1);
		kozepre = 0;
		break;
	    case 3:
		igazitas = -(COLS - 1);
		break;
	}
    }
    ide = malloc(abs(igazitas) + 1);
    strncpy(ide, s, abs(igazitas));
    ide[abs(igazitas)] = 0;
    if ((i = strlen(ide)) < abs(igazitas)) {
	if (kozepre) {
	    x = abs(igazitas) - i;
	    y = x / 2;
	    x -= y;
	    if (igazitas < 0 && x != y) {
		c = x;
		x = y;
		y = c;
	    }
	    while (x)
		ide[y + i + (--x)] = 32;
	    for (i--; i >= 0; i--)
		ide[i + y] = ide[i];
	    while (y)
		ide[--y] = 32;
	}
	else if (igazitas > 0) {
	    for (; i < igazitas; i++)
		ide[i] = 32;
	    ide[i] = 0;
	}
	else {
	    x = -igazitas - i;
	    for (; i >= 0; i--)
		ide[i + x] = ide[i];
	    while (x)
		ide[--x] = 32;
	}
    }
    return (ide);
}
int strwnum(char *s)
{
    int i = 0, x = 0, inword = 0, h = 0;

    for (; *s; s++) {
	if (*s == '"')
	    h ^= 1;
	if ((!inword) && (!isspace(*s))) {
	    inword = 1;
	    x++;
	}
	if ((inword) && (isspace(*s)))
	    if (!h)
		inword = 0;
    }
    return (x);
}
void quote(char *p)
{
    char *s = p, *q;
    int sl = 0;

    if (!s)
	return;
    while (*s) {
	if (*s == '"' || *s == '\\') {
	    q = s + strlen(s) + 1 + sl;
	    for (; q != (s + sl); q--)
		*q = *(q - 1 - sl);
	    *s = '\\';
	    s++;
	    if (sl) {
		*(s + 1) = '\\';
		s++;
	    }
	}
	if (sl)
	    sl = 0;
	s++;
    }
    return;
}
int dollar(char **s, char *ide)
{
    char temp[1024] = "";
    char *p = *s, *t, *q;
    int i = 0, igazitas = 0, modosito = 'x', kozepre = 0;
    int elso, utolso;
    int x, fve = 1;
    int dquote = dontquote;
    struct valtozo *var;
    struct kif *kif;
    int save_varflag = varflag;

#ifdef AKII_DEBUG
    dbgenter(dbg_dollar, *s);
#endif
    dontquote = 0;
    errno = 0;
    t = ++p;
    if (*t == '$') {
	*s = ++p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	*ide = '$';
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (1);
    }
    if (*t == '<') {
	p++;
	if (*p == '~') {
	    p++;
	    kozepre = 1;
	}
	if (*p == '-')
	    kozepre += 2;
	igazitas = strtol(p, &t, 10);
	p = t;
	if (*t != '>') {
	    strcpy(errtemp, (*s));
	    *s = p;
	    errno = ENOCSUCSOS;
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (-i - 1);
	}
	if (!igazitas) {
	    igazitas = 1;
	    kozepre += 10;
	}
	p++;
	t++;
    }
    else if ((*t == '?') || (*t == '/')) {
	*s = ++p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	sprintf(ide, "%d", (*t == '?') ? kerdojel : mkerdojel);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    else if (*t == '#') {
	modosito = '#';
	p++;
	t++;
    }
    else if (*t == '@') {
	modosito = '@';
	p++;
	t++;
    }
    else if (*t == '!') {
	modosito = '!';
	p++;
	t++;
    }
    if (*t == '^') {
        if (blockname)
            strcpy(ide,blockname);
        t++;
        (*s)=t;
        return(strlen(ide));
    }
    if ((*t == '%') || (*t == '&') || (*t == ':')) {
	x = *t;
	if (*++p == '(') {
	    p++;
	    braces(&p, temp, '(', ')');
	    if (*p++ != ')') {
		errno = ENOZAROJEL;
		strcpy(errtemp, *s);
		*s = p;
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	    t = parsedollar(temp);
	    if (errno) {
		free(t);
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	    strcpy(temp, t);
	    free(t);
	    t = temp;
	    t = dollarnum(&t, (x == '%') ? 1 : ((x == '&') ? 2 : 3));
	    if (errno) {
		free(t);
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	}
	else {
	    t = dollarnum(&p, (x == '%') ? 1 : ((x == '&') ? 2 : 3));
	    if (errno) {
		free(t);
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	}
	*s = p;
	if (!ide) {
	    free(t);
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	strcpy(ide, t);
	free(t);
	if (igazitas) {
	    t = igazit(ide, igazitas, kozepre);
	    strcpy(ide, t);
	    free(t);
	}
	if (modosito == '@')
	    sprintf(temp, "%d", strlen(ide));
	else if (modosito == '#')
	    sprintf(temp, "%d", strwnum(ide));
	else {
	    if (modosito != '!' && !dquote)
		quote(ide);
	    strcpy(temp, ide);
	}
	strcpy(ide, temp);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    else if (*t == '~') {
	*s = ++p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	sprintf(temp, "%d", (numvarlist) ? (int) (numvarlist[0]) : 0);
	if (igazitas) {
	    t = igazit(temp, igazitas, kozepre);
	    strcpy(ide, t);
	    free(t);
	}
	else
	    strcpy(ide, temp);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    else if (*t == '*') {
	*s = ++p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	t = getnumvar(numvarlist, 1, -1);
	if (igazitas) {
	    p = t;
	    t = igazit(p, igazitas, kozepre);
	    free(p);
	}
	if (modosito == '@')
	    sprintf(temp, "%d", strlen(t));
	else if (modosito == '#')
	    sprintf(temp, "%d", strwnum(t));
	else {
	    strcpy(temp, t);
	    if (modosito != '!' && !dquote)
		quote(temp);
	    free(t);
	    t = NULL;
	}
	strcpy(ide, temp);
	if (t)
	    free(t);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    else if ((*t == '-') || (isdigit(*t))) {
	t = dollarnum(&p, 0);
	*s = p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	if (igazitas) {
	    p = t;
	    t = igazit(p, igazitas, kozepre);
	    free(p);
	}
	if (modosito == '@')
	    sprintf(temp, "%d", strlen(t));
	else if (modosito == '#')
	    sprintf(temp, "%d", strwnum(t));
	else {
	    strcpy(temp, t);
	    if (modosito != '!' && !dquote)
		quote(temp);
	}
	strcpy(ide, temp);
	free(t);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    if (*t == '{') {
	p++;
	braces(&p, temp, '{', '}');
	if (*p++ != '}') {
	    strcpy(errtemp, (*s));
	    *s = p;
	    errno = ENOZAROKAPOCS;
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (-i - 1);
	}
	t = temp;
	*s = p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	kif = readkif(&t);
	if (!kif) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (-1);
	}
	t = dokif(kif);
	if (!t) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (-1);
	}
	freekiffa(kif);
	if (igazitas) {
	    t = igazit(q = t, igazitas, kozepre);
	    free(q);
	}
	if (modosito == '@')
	    sprintf(temp, "%d", strlen(t));
	else if (modosito == '#')
	    sprintf(temp, "%d", strwnum(t));
	else {
	    strcpy(temp, t);
	    if (modosito != '!' && !dquote)
		quote(temp);
	}
	free(t);
	strcpy(ide, temp);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    else if ((isalpha(*t)) || (*t == '(')) {
	if (*t == '(') {
	    p++;
	    braces(&p, temp, '(', ')');
	    if (*p++ != ')') {
		strcpy(errtemp, (*s));
		*s = p;
		errno = ENOZAROJEL;
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-i - 1);
	    }
	    if (!ide) {
		t = p;
		*s = p;
		while ((*t == '[') && (*t)) {
		    t++;
		    braces(&t, temp, '[', ']');
		    if (*t != ']') {
			errno = ENOZAROSZOGL;
			strcpy(errtemp, *s);
#ifdef AKII_DEBUG
			dbgleave(dbg_dollar);
#endif
			return (1);
		    }
		    t++;
		}
		if (*t == '.') {
		    t++;
		    getvarifp(NULL, &t);
		}
		if (errno) {
#ifdef AKII_DEBUG
		    dbgleave(dbg_dollar);
#endif
		    return (1);
		}
		*s = t;
		while ((isspace(*t)) && (*t))
		    t++;
		if (*t == '(') {
		    t++;
		    *s = t;
		    braces(&t, temp, '(', ')');
		    if (*t != ')') {
			errno = ENOZAROJEL;
			strcpy(errtemp, *s);
		    }
		    else
			*s = t + 1;
		}
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (1);
	    }
	    if (ide) {
		q = parsedollar(temp);
		if (errno) {
		    free(q);
#ifdef AKII_DEBUG
		    dbgleave(dbg_dollar);
#endif
		    return (-1);
		}
		strcpy(temp, q);
		free(q);
	    }
	    t = temp;
	    if (ide) {
		if ((*t == '-') || (isdigit(*t))) {
		    t = dollarnum(&t, 0);
		    *s = p;
		    if (igazitas) {
			p = t;
			t = igazit(p, igazitas, kozepre);
			free(p);
		    }
		    if (modosito == '@')
			sprintf(temp, "%d", strlen(t));
		    else if (modosito == '#')
			sprintf(temp, "%d", strwnum(t));
		    else {
			strcpy(temp, t);
			if (modosito != '!' && !dquote)
			    quote(temp);
		    }
		    strcpy(ide, temp);
		    free(t);
#ifdef AKII_DEBUG
		    dbgleave(dbg_dollar);
#endif
		    return (strlen(ide));
		}
		var = getvar(&varlist, &t, 0, 1);
		if (var) {
		    if (var->tipus == VAR_STRUCT) {
			if ((*p == '.') || (*p == '['))
			    var = getvar(&var, &p, 0, 1);
		    }
		}
		else if ((*p == '.') || (*p == '[')) {
		    t = temp;
		    varflag = VFLAG_NORMAL;
		    var = getvar(&varlist, &t, 1, 1);
		    varflag = save_varflag;
		    if (var) {
			if (var->tipus == VAR_STRUCT)
			    if ((*p == '.') || (*p == '['))
				var = getvar(&var, &p, 0, 1);
		    }
		}
	    }
	}
	else
	    var = getvar((ide) ? &varlist : NULL, &p, 0, 1);
	if ((ide) && (!var)) {
	    errno = ENOVAR;
	    strcpy(errtemp, *s);
	    *s = p;
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (-1);
	}
	t = p;
	if (*t && *t == '\\' && *(t + 1) == '(') {
	    for (; *t; t++)
		*t = *(t + 1);
	    t = p;
	    fve = 0;
	}
	if (*p == '(' && fve) {
	    if (ide)
		if ((var->tipus != VAR_FV) && (var->tipus != VAR_ALIAS)) {
		    strncpy(errtemp, *s, p - (*s));
		    errtemp[p - (*s)] = 0;
		    *s = p;
		    errno = EEZNEMFV;
#ifdef AKII_DEBUG
		    dbgleave(dbg_dollar);
#endif
		    return (-1);
		}
	    p++;
	    braces(&p, temp, '(', ')');
	    if (*p != ')') {
		strcpy(errtemp, (*s));
		*s = p;
		errno = ENOZAROJEL;
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	    p++;
	    if (!ide) {
		*s = p;
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (1);
	    }
	    q = parsedollar(temp);
	    if (errno) {
		free(q);
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	    strcpy(temp, q);
	    free(q);
	    q = callfv(var, temp);
	    strcpy(temp, q);
	    *s = p;
	    if (igazitas) {
		t = igazit(temp, igazitas, kozepre);
		strcpy(temp, t);
		free(t);
	    }
	    if (modosito == '@')
		sprintf(temp, "%d", strlen(temp));
	    else if (modosito == '#')
		sprintf(temp, "%d", strwnum(temp));
	    else if (modosito != '!' && !dquote)
		quote(temp);
	    strcpy(ide, temp);
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (strlen(ide));
	}
	else if (ide) {
	    if ((var->tipus == VAR_NORMAL) || ((var->tipus == VAR_STRUCT) && (modosito == '#'))) {
		if (!ide) {
		    *s = p;
#ifdef AKII_DEBUG
		    dbgleave(dbg_dollar);
#endif
		    return (1);
		}
		if (var->tipus == VAR_NORMAL)
		    strcpy(temp, var->ertek);
	    }
	    else {
		strncpy(errtemp, *s, p - (*s));
		errtemp[p - (*s)] = 0;
		*s = p;
		if (var->tipus == VAR_VEZER)
		    errno = EEZVEZER;
		else if (var->tipus == VAR_STRUCT)
		    errno = ESTRUCT;
		else
		    errno = EEZFV;
#ifdef AKII_DEBUG
		dbgleave(dbg_dollar);
#endif
		return (-1);
	    }
	}
	*s = p;
	if (!ide) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_dollar);
#endif
	    return (1);
	}
	if (igazitas) {
	    t = igazit(temp, igazitas, kozepre);
	    strcpy(temp, t);
	    free(t);
	}
	if (modosito == '@')
	    sprintf(temp, "%d", strlen(temp));
	if (modosito == '#') {
	    if (var->tipus == VAR_NORMAL)
		sprintf(temp, "%d", strwnum(temp));
	    else {
		x = 0;
		var = (struct valtozo *) var->ertek;
		while (var) {
		    x++;
		    var = var->next;
		}
		sprintf(temp, "%d", x);
	    }
	}
	else if (modosito != '!' && !dquote)
	    quote(temp);
	strcpy(ide, temp);
#ifdef AKII_DEBUG
	dbgleave(dbg_dollar);
#endif
	return (strlen(ide));
    }
    strcpy(errtemp, (*s));
    *s = p;
    errno = ENOSEMMI;
#ifdef AKII_DEBUG
    dbgleave(dbg_dollar);
#endif
    return (-1);
}
char *getnumvar(char **itt, int elso, int utolso)
{
    char temp[1024] = "";
    char *q;
    int i, x;

#ifdef AKII_DEBUG
    dbgenter(dbg_getnumvar, "?");
#endif
    if (elso == 0)
	elso = 1;
    if (utolso == 0)
	utolso = 1;
    if (!itt) {
	q = malloc(1);
	*q = 0;
#ifdef AKII_DEBUG
	dbgleave(dbg_getnumvar);
#endif
	return (q);
    }
    i = (int) itt[0];
    if (((elso == utolso) && (i < utolso)) || (elso > i)) {
	q = malloc(1);
	*q = 0;
#ifdef AKII_DEBUG
	dbgleave(dbg_getnumvar);
#endif
	return (q);
    }
    if ((utolso < 0) || (utolso > i))
	utolso = i;
    if (elso > utolso)
	elso = utolso;
    i = 0;
    for (x = elso; x <= utolso; x++) {
	strcpy(temp + i, itt[x]);
	i = strlen(temp);
	temp[i++] = 32;
    }
    if (i > 0)
	temp[i - 1] = 0;
    q = malloc(i + 1);
    strcpy(q, temp);
#ifdef AKII_DEBUG
    dbgleave(dbg_getnumvar);
#endif
    return (q);
}
int istrcmp(char *egyik, char *masik)
{
    while (*egyik && *masik) {
	if (toupper(*egyik) != toupper(*masik))
	    return (toupper(*egyik) - toupper(*masik));
	egyik++;
	masik++;
    }
    return (toupper(*egyik) - toupper(*masik));
}
int istrncmp(char *egyik, char *masik, int i)
{
    while ((*egyik && *masik) && i--) {
	if (toupper(*egyik) != toupper(*masik))
	    return (toupper(*egyik) - toupper(*masik));
	egyik++;
	masik++;
    }
    if (i)
	return (toupper(*egyik) - toupper(*masik));
    return (0);
}
struct valtozo *getvarifp(struct valtozo **hol, char **s)
{
    struct valtozo *v;
    char *p = *s, temp[1024];
    int voltkapocs = 0;

#ifdef AKII_DEBUG
    dbgenter(dbg_getvarifp, *s);
#endif
    while ((isalnum(*p)) || (*p == '_') || (*p == '.') || (*p == '[')) {
	while ((isalnum(*p) || (*p == '_')) && (*p))
	    p++;
	if (*p == '.') {
	    p++;
	    voltkapocs = 1;
	    continue;
	}
	if (*p == '[') {
	    p++;
	    braces(&p, temp, '[', ']');
	    voltkapocs = 1;
	    if (*p != ']') {
		errno = ENOZAROSZOGL;
		strcpy(errtemp, *s);
#ifdef AKII_DEBUG
		dbgleave(dbg_getvarifp);
#endif
		return (NULL);
	    }
	    p++;
	    continue;
	}
    }
    if ((hol) && (!voltkapocs)) {
	v = getvar(hol, s, 0, 0);
	if (!v)
	    errno = 0;
#ifdef AKII_DEBUG
	dbgleave(dbg_getvarifp);
#endif
	return (v);
    }
    else {
	*s = p;
#ifdef AKII_DEBUG
	dbgleave(dbg_getvarifp);
#endif
	return (NULL);
    }
}
struct valtozo *getvar(struct valtozo **hol, char **s, int makeifnotexist, int lehetstruct)
{
    struct valtozo *v, *v2, *var = NULL;
    char temp[1024], *p = *s, *t = temp;
    char *q;
    char nevutani = 32;
    int i = 0, newv = 0;
    int save_varflag = varflag;

#ifdef AKII_DEBUG
    dbgenter(dbg_getvar, *s);
#endif
    errno = 0;
    if (hol == &varlist) {
	var = getvar(&localvars[locallevel], &p, 0, lehetstruct);
	if (errno == ENOSTRUCTELEM) {
	    p = *s;
	    varflag = VFLAG_LOCAL;
	    var = getvar(&localvars[locallevel], &p, 1, lehetstruct);
	    varflag = save_varflag;
	}
	if (var) {
	    *s = p;
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (var);
	}
	if (errno == ENOVAR)
	    errno = 0;
	if (errno) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
    }
    p = *s;
    if ((*p != '[') && (*p != '.')) {
	while (*p) {
	    if (isalnum(*p) || (*p == '_')) {
		temp[i++] = toupper(*p);
		p++;
	    }
	    else
		break;
	}
	nevutani = *p;
	temp[i] = 0;
	if (!i) {
	    *s = p;
	    strcpy(errtemp, *s);
	    errno = ENULLVAR;
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
	if (hol) {
	    v = *hol;
	    v2 = NULL;
	    while (v) {
		if ((i = istrcmp(v->nev, temp)) == 0) {
		    var = v;
		    break;
		}
		else if (i > 0)
		    break;
		v2 = v;
		v = v->next;
	    }
	    if ((!var) && (hol == &varlist)) {
		var = getvar(&bivarlist, s, 0, lehetstruct);
		if (var) {
#ifdef AKII_DEBUG
		    dbgleave(dbg_getvar);
#endif
		    return (var);
		}
		else if (errno != ENOVAR) {
#ifdef AKII_DEBUG
		    dbgleave(dbg_getvar);
#endif
		    return (NULL);
		}
		errno = 0;
	    }
	    if (var)
		while (var->tipus == VAR_FOREACH) {
		    var = (struct valtozo *) var->ertek;
		    makeifnotexist = 1;
		}
	    *s = p;
	    if ((!var) && (!makeifnotexist)) {
		strcpy(errtemp, temp);
		if ((hol) && (hol != &bivarlist) && (hol != &localvars[locallevel]) && (hol != &varlist))
		    errno = ENOSTRUCTELEM;
		else
		    errno = ENOVAR;
		/*ha tombnek kepzelte a felhasznalo, de meg nem letezik, akkor beolvassuk a megfelelo karakterig: */
		if (nevutani == '[' || nevutani == '.') {
		    getvarifp(NULL, s);
		}
#ifdef AKII_DEBUG
		dbgleave(dbg_getvar);
#endif
		return (NULL);
	    }
	    if (!var) {
		newv = makeifnotexist;
		if (!*hol) {	/*ha ures a valtozosor (elso valtozo) */
		    v = malloc(sizeof(struct valtozo));

		    v->tipus = VAR_NORMAL;
		    v->nev = malloc(strlen(temp) + 1);
		    strcpy(v->nev, temp);
		    v->ertek = malloc(1);
		    *(v->ertek) = 0;
		    v->prev = v->next = NULL;
		    *hol = v;
		    v->parent = hol;
		    v->notdeletable = 0;
		    var = v;
		    var->flag = varflag;
		}
		else if (!v) {	/*nem ures a sor, es a legegehez kell fuzni */
		    v = v2->next = malloc(sizeof(struct valtozo));

		    v->prev = v2;
		    v->tipus = VAR_NORMAL;
		    v->nev = malloc(strlen(temp) + 1);
		    strcpy(v->nev, temp);
		    v->ertek = malloc(1);
		    *(v->ertek) = 0;
		    v->next = NULL;
		    v->parent = NULL;
		    v->notdeletable = 0;
		    var = v;
		    var->flag = varflag;
		}
		else {		/*kozbe kell szurni */
		    var = v2;
		    v2 = malloc(sizeof(struct valtozo));

		    if (v->parent) {
			v2->parent = v->parent;
			*hol = v2;
			v->parent = NULL;
		    }
		    else
			v2->parent = NULL;
		    v2->next = v;
		    if (var) {
			var->next = v2;
		    }
		    v2->prev = var;
		    v2->tipus = VAR_NORMAL;
		    v2->nev = malloc(strlen(temp) + 1);
		    strcpy(v2->nev, temp);
		    v2->ertek = malloc(1);
		    *(v2->ertek) = 0;
		    v2->notdeletable = 0;
		    var = v2;
		    var->flag = varflag;
		    v->prev = var;
		}
	    }
	}
	else
	    *s = p;
    }
    else {
	var = *hol;
    }
    while ((*p == '[') || (*p == '.')) {
	if (hol) {
	    if (var->tipus != VAR_STRUCT) {
		if ((!newv) && ((var->tipus == VAR_NORMAL) && var->ertek[0])) {
		    errno = ENONSTRUCT;
		    strcpy(errtemp, temp);
		    *s = p;
#ifdef AKII_DEBUG
		    dbgleave(dbg_getvar);
#endif
		    return (NULL);
		}
		newv = 1;
		var->tipus = VAR_STRUCT;
		free(var->ertek);
		var->ertek = NULL;
	    }
	}
	if (*p == '.') {
	    *s = ++p;
	    var = getvar((hol) ? (struct valtozo **) &var->ertek : NULL, s, makeifnotexist, lehetstruct);
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (var);
	}
	*s = p;
	p++;
	braces(&p, temp, '[', ']');
	var->notdeletable++;
	if (hol) {
	    q = parsedollar(temp);
	    if (errno) {
		free(q);
#ifdef AKII_DEBUG
		dbgleave(dbg_getvar);
#endif
		return (NULL);
	    }
	    strcpy(temp, q);
	    free(q);
	}
	var->notdeletable--;
	if (*p != ']') {
	    strcpy(errtemp, (*s));
	    *s = p;
	    errno = ENOZAROSZOGL;
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
	*s = ++p;
	if (!hol)
	    continue;
	t = temp;
	if (!var) {
	    errno = ENOVAR;
	    errtemp[0] = 0;
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
	var = getvar((struct valtozo **) &(var->ertek), &t, makeifnotexist, 1);
	if (!var) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
	if ((var->tipus == VAR_NORMAL) && (*(var->ertek) == 0) && (hol != &bivarlist))
	    newv = makeifnotexist;
	else
	    newv = 0;
    }
    if (hol)
	if ((!lehetstruct) && (var->tipus == VAR_STRUCT)) {
	    strcpy(errtemp, temp);
	    errno = ESTRUCT;
#ifdef AKII_DEBUG
	    dbgleave(dbg_getvar);
#endif
	    return (NULL);
	}
#ifdef AKII_DEBUG
    dbgleave(dbg_getvar);
#endif
    return (var);
}
void freeonlist(struct onstr *on)
{
    struct onstr *n;
    int i;

#ifdef AKII_DEBUG
    dbgenter(dbg_freeonlist, "?");
#endif
    while (on) {
	if (on->nev)
	    free(on->nev);
	freepar(on->par);
	for (i = 0; i < on->kifek; i++)
	    free(on->regkifek[i]);
	if (on->regkifek)
	    free(on->regkifek);
	n = on;
	free(on);
	on = n->next;
    }
#ifdef AKII_DEBUG
    dbgleave(dbg_freeonlist);
#endif
    return;
}
int freevarlist(struct valtozo *hol, int action)
{
    struct valtozo *v;
    struct aliasstr *ali;
    int i = 0;

#ifdef AKII_DEBUG
    dbgenter(dbg_freevarlist, "?");
#endif
    if (!hol) {
#ifdef AKII_DEBUG
	dbgleave(dbg_freevarlist);
#endif
	return (i);
    }
    if (hol->notdeletable) {
#ifdef AKII_DEBUG
	dbgleave(dbg_freevarlist);
#endif
	return (1);
    }
    switch (hol->tipus) {
	case VAR_NORMAL:
	    free(hol->ertek);
	    break;
	case VAR_STRUCT:
	    i = freevarlist((struct valtozo *) (hol->ertek), 1);
	    if (i) {
#ifdef AKII_DEBUG
		dbgleave(dbg_freevarlist);
#endif
		return (i);
	    }
	    break;
	case VAR_ALIAS:
	    ali = (struct aliasstr *) (hol->ertek);
	    freepar(ali->par);
	    for (i = 0; i < ali->cpnum; i++)
		free(ali->cps[i]);
	    if (ali->cps)
		free(ali->cps);
	    i = 0;
	    free(ali);
	    break;
	case VAR_FV:
	case VAR_VEZER:
	case VAR_FOREACH:
	    break;
    }
    if (action) {
	if (hol->next)
	    i = freevarlist(hol->next, action);
	if (i) {
#ifdef AKII_DEBUG
	    dbgleave(dbg_freevarlist);
#endif
	    return (1);
	}
	if (hol->parent)
	    *hol->parent = NULL;
	free(hol->nev);
	free(hol);
    }
    else {
	free(hol->nev);
	v = hol;
	if (hol->next) {
	    hol->next->prev = hol->prev;
	    if (hol->prev)
		hol->prev->next = hol->next;
	    else {
		hol->next->parent = hol->parent;
		*hol->parent = hol->next;
	    }
	}
	if (hol->prev)
	    hol->prev->next = hol->next;
	else {
	    if (!hol->next) {
		*hol->parent = NULL;
	    }
	}
	free(v);
    }
#ifdef AKII_DEBUG
    dbgleave(dbg_freevarlist);
#endif
    return (0);
}
void freenumvarlist(char ***s)
{
    int x;

#ifdef AKII_DEBUG
    dbgenter(dbg_freenumvarlist, "?");
#endif
    if (!s) {
#ifdef AKII_DEBUG
	dbgleave(dbg_freenumvarlist);
#endif
	return;
    }
    if (!*s) {
#ifdef AKII_DEBUG
	dbgleave(dbg_freenumvarlist);
#endif
	return;
    }
    x = (int) (**s);
    for (; x > 0; x--)
	free(*((*s) + x));
    free(*s);
    *s = NULL;
#ifdef AKII_DEBUG
    dbgleave(dbg_freenumvarlist);
#endif
    return;
}
void freetimerlist(void)
{
    struct timerstr *t;

#ifdef AKII_DEBUG
    dbgenter(dbg_freetimerlist, "?");
#endif
    while (timerlist) {
	if (timerlist->nev)
	    free(timerlist->nev);
	freepar(timerlist->par);
	t = timerlist;
	timerlist = timerlist->next;
	free(t);
    }
#ifdef AKII_DEBUG
    dbgleave(dbg_freetimerlist);
#endif
    return;
}
void newvezer(char *s)
{
    struct valtozo *var;

    var = getvar(&bivarlist, &s, 1, 0);
    free(var->ertek);
    var->tipus = VAR_VEZER;
    return;
}
void initvariables(char *server, int port, char *nev, char *verinfo, char *ver, int build)
{
    struct valtozo *var;
    char temp[256], *t = temp, *p;
    int save_varflag = varflag;

    memset(localvars, 0, sizeof(localvars));
    memset(use_list, 0, sizeof(use_list));
    use_list[0] = strdup("belep");
    use_list[1] = strdup("ping");
    use_list[2] = strdup("stat");
    use_list[3] = strdup("vege");
    use_list[4] = strdup("who");
    usenum = 4;

    varflag = VFLAG_BELSO;

    strcpy(t, "akii.Szervernev");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, server);
    t = temp;

    strcpy(t, "akii.Szerverport");
    var = getvar(&bivarlist, &t, 1, 0);
    sprintf(temp, "%d", port);
    setvar(var, temp);
    t = temp;

    strcpy(t, "akii.Csatorna");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, "@");
    akii_csatorna = var;
    t = temp;

    strcpy(t, "akii.termwidth");
    var = getvar(&bivarlist, &t, 1, 0);
    setivar(var, COLS);
    akii_termwidth = var;
    t = temp;
    strcpy(t, "akii.termheight");
    var = getvar(&bivarlist, &t, 1, 0);
    setivar(var, LINES);
    akii_termheight = var;
    t = temp;

    strcpy(t, "akii.nick");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, nev);
    akii_nick = var;
    t = temp;

    strcpy(t, "akii.idle");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, "0");
    akii_idle = var;
    t = temp;
    strcpy(t, "akii.inputidle");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, "0");
    akii_inputidle = var;
    t = temp;

    strcpy(t, "akii.winsize");
    var = getvar(&bivarlist, &t, 1, 0);
    setivar(var, winsize);
    akii_winsize = var;
    t = temp;

    strcpy(t, "akii.statusz");
    var = getvar(&bivarlist, &t, 1, 0);
    if (dbgfp) {
	sprintf(temp, " [%%t/%%e]  Nezd: '/echo $$fv()'%%>Log: %%l  /%%i/  akII R%s #%d /DBG/ ", ver, build);
	setvar(var, temp);
    }
    else {
	sprintf(temp, " [%%t/%%e]  Nezd: '/echo $$fv()'%%>Log: %%l  /%%i/  akII R%s #%d ", ver, build);
	setvar(var, temp);
    }
    t = temp;

    strcpy(t, "akii.winstatusz");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, " %w %~Nezd: '/list akii' %>%s sor, %o/%a oldal, $akii.winsize sor/oldal ");
    t = temp;

    strcpy(t, "akii.Verzio");
    var = getvar(&bivarlist, &t, 1, 0);
    p = strchr(verinfo, '\n');
    if (p)
	*p = 32;
    setvar(var, verinfo);
    akii_verzio = var;
    if (p)
	*p = '\n';
    t = temp;

    strcpy(t, "akii.RETURN");
    var = getvar(&bivarlist, &t, 1, 0);
    t = temp;

    strcpy(t, "akii.novice");
    var = getvar(&bivarlist, &t, 1, 0);
    akii_novice = var;
    setvar(var, "1");
    t = temp;

    strcpy(t, "akii.sortby");
    var = getvar(&bivarlist, &t, 1, 0);
    akii_sortby = var;
    setvar(var, "Nick");
    t = temp;

    strcpy(t, "akii.nocs");
    var = getvar(&bivarlist, &t, 1, 0);
    akii_nocs = var;
    setivar(var, nocs);
    t = temp;

    strcpy(t, "akii.usebold");
    var = getvar(&bivarlist, &t, 1, 0);
    akii_usebold = var;
    setvar(var, "1");
    t = temp;

    strcpy(t, "akii.usewin");
    var = getvar(&bivarlist, &t, 1, 0);
    akii_usewin = var;
    setivar(var, usewin);
    t = temp;

    strcpy(t, "akii.timeout");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, "1000");
    akii_timeout = var;
    t = temp;

    strcpy(t, "akii.floodprotect");
    var = getvar(&bivarlist, &t, 1, 0);
    setivar(var, floodprotect);
    akii_floodprotect = var;
    t = temp;

    strcpy(t, "akii.prompt");
    var = getvar(&bivarlist, &t, 1, 0);
    setvar(var, "%n on %c:> ");
    akii_prompt = var;
    t = temp;

    newvezer("on");
    newvezer("onsend");
    newvezer("if");
    newvezer("else");
    newvezer("for");
    newvezer("do");
    newvezer("while");
    newvezer("foreach");
    newvezer("break");
    newvezer("continue");
    newvezer("alias");
    newvezer("timer");
    newvezer("return");
    newvezer("checkpoint");
    newvezer("escape");
    newvezer("nomoreon");
    newvezer("nexton");

    varflag = save_varflag;

    return;
}
void freefilelist(void)
{
    struct filelist *fl;

    while (flist) {
	free(flist->fnev);
	fclose(flist->fp);
	fl = flist;
	flist = fl->next;
	free(fl);
    }
    return;
}
void freemsglist(void)
{
    struct msgs *x = msgk;

    while (x) {
	msgk = msgk->next;
	free(x);
	x = msgk;
    }
    return;
}
void donevariables_fvs(void)
{
    freevarlist(varlist, 1);
    freevarlist(bivarlist, 1);
    freenumvarlist(&numvarlist);
    freenumvarlist(&chvarlist);
    freenumvarlist(&mchvarlist);
    freeonlist(onlist);
    freeonlist(onsendlist);
    freetimerlist();
    freefilelist();
    freemsglist();
    return;
}
struct valtozo *setvar(struct valtozo *ezt, char *erre)
{
#ifdef AKII_DEBUG
    dbgenter(dbg_setvar, ezt->nev);
#endif
    if (!ezt) {
#ifdef AKII_DEBUG
	dbgleave(dbg_setvar);
#endif
	return (NULL);
    }
    if (ezt->tipus != VAR_NORMAL) {
#ifdef AKII_DEBUG
	dbgleave(dbg_setvar);
#endif
	return (NULL);
    }
    if (ezt == akii_verzio || ezt == akii_nick || ezt == akii_csatorna || ezt == akii_termwidth || ezt == akii_termheight || ezt == akii_idle || ezt == akii_inputidle) {
#ifdef AKII_DEBUG
	dbgleave(dbg_setvar);
#endif
	return (NULL);
    }
    if (ezt->ertek)
	free(ezt->ertek);
    ezt->ertek = malloc(strlen(erre) + 1);
    strcpy(ezt->ertek, erre);
    checkchanges(ezt);
#ifdef AKII_DEBUG
    dbgleave(dbg_setvar);
#endif
    return (ezt);
}
struct valtozo *setnvar(struct valtozo *ezt, char *erre, int ennyit)
{
#ifdef AKII_DEBUG
    dbgenter(dbg_setnvar, ezt->nev);
#endif
    if (!ezt) {
#ifdef AKII_DEBUG
	dbgleave(dbg_setnvar);
#endif
	return (NULL);
    }
    if (ezt->tipus != VAR_NORMAL) {
#ifdef AKII_DEBUG
	dbgleave(dbg_setnvar);
#endif
	return (NULL);
    }
    if (ezt->ertek)
	free(ezt->ertek);
    ezt->ertek = malloc(ennyit + 1);
    strncpy(ezt->ertek, erre, ennyit);
    ezt->ertek[ennyit] = 0;
    checkchanges(ezt);
#ifdef AKII_DEBUG
    dbgleave(dbg_setnvar);
#endif
    return (ezt);
}
struct valtozo *setivar(struct valtozo *ezt, long int erre)
{
    static char temp[25];

    if (!ezt)
	return (NULL);
    if (ezt->tipus != VAR_NORMAL)
	return (NULL);
    if (ezt->ertek)
	free(ezt->ertek);
    sprintf(temp, "%ld", erre);
    ezt->ertek = malloc(strlen(temp) + 1);
    strcpy(ezt->ertek, temp);
    checkchanges(ezt);
    return (ezt);
}
struct valtozo *newfv(char *ezt, void (*fv) (struct valtozo *))
{
    struct valtozo *var;

    if (!ezt)
	return (NULL);
    var = getvar(&bivarlist, &ezt, 1, 0);
    if (var) {
	if (var->tipus != VAR_NORMAL)
	    return (NULL);
	if (var->ertek[0] != 0)
	    return (NULL);
	free(var->ertek);
	var->ertek = (char *) fv;
	var->tipus = VAR_FV;
    }
    else {
	return (NULL);
    }
    return (var);
}
struct valtozo *getvar2(char *nev)
{
    struct valtozo *var;

    var = getvar(&varlist, &nev, 1, 0);
    if (var)
	if (var->tipus != VAR_NORMAL)
	    var = NULL;
    return (var);
}
int adduse(char *s)
{
    int i;

    for (i = 0; i < usenum; i++) {
	if (istrcmp(use_list[i], s) == 0)
	    return (0);
    }
    if (usenum >= 100)
	return (-1);
    use_list[usenum] = strdup(s);
    usenum++;
    return (usenum);
}
int inuse(char *s)
{
    int i;

    for (i = 0; i < usenum; i++) {
	if (istrcmp(use_list[i], s) == 0)
	    return (1);
    }
    return (0);
}
int deluse(char *s)
{
    int i, j;

    for (i = 0; i < usenum; i++) {
	if (istrcmp(use_list[i], s) == 0) {
	    free(use_list[i]);
	    usenum--;
	    for (j = i; j < usenum; j++)
		use_list[j] = use_list[j + 1];
	    return (1);
	}
    }
    return (0);
}
